/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package  {
	
	import com.dncompute.graphics.ArrowStyle;
	import com.dncompute.graphics.LineArrow;
	import flash.display.*;
	import flash.events.*;
	import flash.filters.DropShadowFilter;
	import flash.geom.*;
	import flash.text.*;
	import flash.ui.*;
	import com.boostworthy.animation.management.*;
	import com.boostworthy.animation.easing.*;
	import com.boostworthy.animation.sequence.*;
	import com.boostworthy.animation.sequence.tweens.*;
	import com.boostworthy.animation.rendering.*;
	import com.boostworthy.core.Global;
	
	/**
	 * Animation for 10,000' overview of DMP
	 * @author Joe Devietti
	 */
	public class QuantumBuilders extends Sprite {
		
		// stage layout
		private const STAGE_WIDTH:int = 600;
		private const PANEL_WIDTH:Number = STAGE_WIDTH / 3.0;
		
		// 3-panel layout
		private const PANEL_START:Array = [0, PANEL_WIDTH, (2*PANEL_WIDTH)];
		
		private const CODE_WIDTH:int = 100;
		private const CODE_GAP:int = (PANEL_WIDTH - CODE_WIDTH) / 2;
		private const CODE_TOP:int = 5;
		private const CODE_SPLIT_VGAP:int = 6;
		private const CODE_ADJACENT_VFUDGE:int = -2;
		
		private const INSN_WIDTH:int = CODE_WIDTH-10;
		private const INSN_HEIGHT:int = 25;
		
		private const LABEL_VOFFSET:int = 17;
		
		private const COMM_COLOR:uint = 0x0066CC;
		
		private function drawCode(insns:Array):Sprite {
			// box
			var t:Sprite = new Sprite();
			t.graphics.lineStyle();
			t.graphics.beginFill( 0xCCCCCC );
			t.graphics.drawRect( 0, 0, CODE_WIDTH, INSN_HEIGHT * insns.length );
			t.graphics.endFill();
			
			var dsf:DropShadowFilter = MyShapes.getDropShadow(0x0);
			dsf.alpha = 0.0; // gets animated in later
			dsf.blurX = 6;
			dsf.blurY = 4;
			t.filters = [ dsf ];
			
			// add insns
			var i:uint = 1;
			for each ( var insn:String in insns ) {
				// create a TextField object
				var txt:TextField = new UnselectableTextField();
				txt.text = insn;
				txt.name = "insn"+(i-1);
				txt.autoSize = TextFieldAutoSize.LEFT;
				
				var format:TextFormat = new TextFormat();
				format.font = "Lucida Console";
				format.color = 0x0;
				format.size = 17;
				format.bold = true;
				txt.setTextFormat( format );
				
				t.addChild( txt );
				
				// center text horizontally within codebox
				var bb:Rectangle = txt.getBounds( t );
				// use coords in "rectangle space"
				txt.x = ((t.width - bb.width) / 2);
				// balance insns vertically
				var vgap:Number = (t.height - (insns.length * txt.height)) / (insns.length+1);
				txt.y = (i * vgap) + ((i-1) * txt.height);
				
				i++;
			}
			
			return t;
		}
		
		private function drawHiliter(size:uint):Sprite {
			var t:Sprite = new Sprite();
			t.graphics.lineStyle();
			t.graphics.beginFill( 0xFFFF00 );
			t.graphics.drawRect( 0, 0, CODE_WIDTH, INSN_HEIGHT * size );
			t.graphics.endFill();
			return t;
		}
		
		/** Fade in the drop shadow filters of each sprite in the list. 
		 * Uses a non-blocking Anim.FADE for timing. */
		private function fadeInDropShadows(sprites:Array):void {
			for each ( var s:Sprite in sprites ) {
				g_timeline.addTween( new FilterTween(s,DropShadowFilter,"alpha",0.8,
					Anim.Start(),Anim.StopNoIncr(Anim.FADE),Transitions.LINEAR) );
			}
		}
		
		private function drawLabel(text:String):TextField {
			// create text
			var tf:TextField = new UnselectableTextField();
			tf.text = text;
			tf.autoSize = TextFieldAutoSize.CENTER;
			
			// format text
			var format:TextFormat = new TextFormat();
			format.align = TextFormatAlign.CENTER;
			format.font = "Tw Cen MT";
			format.color = 0x0;
			format.size = 24;
			format.bold = false;
			tf.setTextFormat( format );
			
			return tf;
		}
		
		public function QuantumBuilders() {
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private const g_timeline:Timeline = new Timeline( RenderMethod.ENTER_FRAME );
		

		private function init(e:Event = null):void {
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			
			SceneManager.setup( g_timeline, stage );
			

SceneManager.addPowerpointClick();
SceneManager.addPowerpointClick();
			
// SHOW SYNC CODE
var sp_codes:Array = [];
var sp_labels:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	var s:Sprite = drawCode( ["acquire","load","release"] );
	s.name = "codeSyncTop";
	s.alpha = 0.0;
	addChild( s );
	sp_codes.push( s );
	
	// center this, as it's the only one at this point
	s.x = PANEL_START[1] + CODE_GAP;
	s.y = CODE_TOP;
	g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	
	var t:Sprite = drawCode( ["load","store","load"] );
	t.name = "codeSyncBottom";
	t.alpha = 0.0;
	addChild( t );
	sp_codes.push( t );
	
	// center this, as it's the only one at this point
	t.x = PANEL_START[1] + CODE_GAP;
	t.y = CODE_TOP + s.height + CODE_ADJACENT_VFUDGE;
	g_timeline.addTween( ExecLayout.fadeToNoIncr(t,1.0) );
	
	Anim.StopIncr(Anim.FADE);
	
	fadeInDropShadows( [s,t] );
	Anim.StopIncr(Anim.FADE);
	
	// add label
	var label:TextField = drawLabel("leverage\nsynchronization");
	label.name = "labelSync";
	label.alpha = 0.0;
	addChild( label );
	sp_labels.push( label );
	
	label.x = PANEL_START[1] + ((PANEL_WIDTH - label.width) / 2);
	label.y = t.y + t.height + LABEL_VOFFSET;
	
	g_timeline.addTween( ExecLayout.fadeToIncr(label,1.0) );
},
function():void {
	for each (var s:DisplayObject in sp_codes) { removeChild(s); }
	sp_codes = [];
	for each (s in sp_labels) { removeChild(s); }
	sp_labels = [];
},
Anim.FADE*3);


// HIGHLIGHT RELEASE INSN
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var top:Sprite = ExecLayout.findByName( sp_codes, "codeSyncTop" );
	var h:Sprite = drawHiliter( 1 );
	h.alpha = 0.0;
	h.name = "hiliter";
	top.addChildAt( h, 0 );
	// coords in code box space
	h.x = 0;
	h.y = 2 * INSN_HEIGHT;
	
	g_timeline.addTween( ExecLayout.fadeToIncr(h,1.0) );
},
function():void {
	var s:Sprite = ExecLayout.findByName( sp_codes, "codeSyncTop" );
	s.removeChild( s.getChildByName("hiliter") );
},
Anim.FADE);


// SPLIT SYNC CODE
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var top:Sprite = ExecLayout.findByName( sp_codes, "codeSyncTop" );
	g_timeline.addTween( new Tween(top,"y",top.y-CODE_SPLIT_VGAP,
		Anim.Start(),Anim.StopNoIncr(Anim.SPLIT_QUANTUM),Transitions.CUBIC_IN_AND_OUT) );
		
	var bottom:Sprite = ExecLayout.findByName( sp_codes, "codeSyncBottom" );
	g_timeline.addTween( new Tween(bottom,"y",bottom.y+CODE_SPLIT_VGAP,
		Anim.Start(),Anim.StopNoIncr(Anim.SPLIT_QUANTUM),Transitions.CUBIC_IN_AND_OUT) );
		
	Anim.StopIncr(Anim.SPLIT_QUANTUM);
},
function():void {},
Anim.SPLIT_QUANTUM);


// SLIDE SYNC STUFF TO THE LEFT AND SHOW BURST
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// slide sync stuff to panel 0
	for each ( var n:String in ["codeSyncTop","codeSyncBottom"] ) {
		var s:Sprite = ExecLayout.findByName( sp_codes, n );
		g_timeline.addTween( new Tween(s,"x",s.x - PANEL_WIDTH,
			Anim.Start(),Anim.StopNoIncr(Anim.SLIDE_CODE),Transitions.CUBIC_IN_AND_OUT) );
	}
	var d:DisplayObject = ExecLayout.findByNameDO( sp_labels, "labelSync" );
	g_timeline.addTween( new Tween(d,"x",d.x - PANEL_WIDTH,
			Anim.Start(),Anim.StopNoIncr(Anim.SLIDE_CODE),Transitions.CUBIC_IN_AND_OUT) );
			
	Anim.StopIncr(Anim.SLIDE_CODE);
	
	// add burst stuff
	
	s = drawCode( ["store","store","store"] );
	s.name = "codeBurstTop";
	s.alpha = 0.0;
	addChild( s );
	sp_codes.push( s );
	
	s.x = PANEL_START[1] + CODE_GAP;
	s.y = CODE_TOP;
	g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	
	var t:Sprite = drawCode( ["load","add","load"] );
	t.name = "codeBurstBottom";
	t.alpha = 0.0;
	addChild( t );
	sp_codes.push( t );
	
	t.x = PANEL_START[1] + CODE_GAP;
	t.y = CODE_TOP + s.height + CODE_ADJACENT_VFUDGE;
	g_timeline.addTween( ExecLayout.fadeToNoIncr(t,1.0) );
	
	Anim.StopIncr(Anim.FADE);
	
	fadeInDropShadows( [s,t] );
	Anim.StopIncr(Anim.FADE);
	
	// add label
	var label:TextField = drawLabel("leverage\ncommunication bursts");
	label.name = "labelBurst";
	label.alpha = 0.0;
	addChild( label );
	sp_labels.push( label );
	
	label.x = PANEL_START[1] + ((PANEL_WIDTH - label.width) / 2);
	label.y = t.y + t.height + LABEL_VOFFSET;
	
	g_timeline.addTween( ExecLayout.fadeToIncr(label,1.0) );
},
function():void {
	sp_codes = ExecLayout.removeByName( ["codeBurstTop","codeBurstBottom"], sp_codes, removeChild );
	sp_labels = ExecLayout.removeByName( ["labelBurst"], sp_labels, removeChild );
},
Anim.SLIDE_CODE + (3*Anim.FADE));


// HIGHLIGHT STORE INSNS
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var top:Sprite = ExecLayout.findByName( sp_codes, "codeBurstTop" );
	var h:Sprite = drawHiliter( 3 );
	h.alpha = 0.0;
	h.name = "hiliter";
	top.addChildAt( h, 0 );
	// coords in code box space
	h.x = 0;
	h.y = 0;
	
	g_timeline.addTween( ExecLayout.fadeToIncr(h,1.0) );
},
function():void {
	var s:Sprite = ExecLayout.findByName( sp_codes, "codeBurstTop" );
	s.removeChild( s.getChildByName("hiliter") );
},
Anim.FADE);


// SPLIT BURST CODE
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var top:Sprite = ExecLayout.findByName( sp_codes, "codeBurstTop" );
	g_timeline.addTween( new Tween(top,"y",top.y-CODE_SPLIT_VGAP,
		Anim.Start(),Anim.StopNoIncr(Anim.SPLIT_QUANTUM),Transitions.CUBIC_IN_AND_OUT) );
		
	var bottom:Sprite = ExecLayout.findByName( sp_codes, "codeBurstBottom" );
	g_timeline.addTween( new Tween(bottom,"y",bottom.y+CODE_SPLIT_VGAP,
		Anim.Start(),Anim.StopNoIncr(Anim.SPLIT_QUANTUM),Transitions.CUBIC_IN_AND_OUT) );
		
	Anim.StopIncr(Anim.SPLIT_QUANTUM);
},
function():void {},
Anim.SPLIT_QUANTUM);


// SHOW SYNC+BURST
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	var s:Sprite = drawCode( ["acquire","load","release"] );
	s.name = "codeSBTop";
	s.alpha = 0.0;
	addChild( s );
	sp_codes.push( s );
	
	s.x = PANEL_START[2] + CODE_GAP;
	s.y = CODE_TOP;
	g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	
	var t:Sprite = drawCode( ["store","store","store"] );
	t.name = "codeSBMiddle";
	t.alpha = 0.0;
	addChild( t );
	sp_codes.push( t );
	
	t.x = PANEL_START[2] + CODE_GAP;
	t.y = CODE_TOP + s.height + CODE_ADJACENT_VFUDGE;
	g_timeline.addTween( ExecLayout.fadeToNoIncr(t,1.0) );
	
	var u:Sprite = drawCode( ["load","add"] );
	u.name = "codeSBBottom";
	u.alpha = 0.0;
	addChild( u );
	sp_codes.push( u );
	
	u.x = PANEL_START[2] + CODE_GAP;
	u.y = t.y + t.height + (2*CODE_ADJACENT_VFUDGE);
	g_timeline.addTween( ExecLayout.fadeToNoIncr(u,1.0) );
	
	Anim.StopIncr(Anim.FADE);
	
	fadeInDropShadows( [s,t,u] );
	Anim.StopIncr(Anim.FADE);
	
	// add label
	var label:TextField = drawLabel("leverage both");
	label.name = "labelSB";
	label.alpha = 0.0;
	addChild( label );
	sp_labels.push( label );
	
	label.x = PANEL_START[2] + ((PANEL_WIDTH - label.width) / 2);
	label.y = u.y + u.height + LABEL_VOFFSET;
	
	g_timeline.addTween( ExecLayout.fadeToIncr(label,1.0) );
},
function():void {
	sp_codes = ExecLayout.removeByName( ["codeSBTop","codeSBMiddle","codeSBBottom"], sp_codes, removeChild );	
	sp_labels = ExecLayout.removeByName( ["labelSB"], sp_labels, removeChild );
},
Anim.FADE*3);


// HIGHLIGHT RELEASE INSN
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var top:Sprite = ExecLayout.findByName( sp_codes, "codeSBTop" );
	var h:Sprite = drawHiliter( 1 );
	h.alpha = 0.0;
	h.name = "hiliter"
	top.addChildAt( h, 0 );
	// coords in code box space
	h.x = 0;
	h.y = 2 * INSN_HEIGHT;
	g_timeline.addTween( ExecLayout.fadeToIncr(h,1.0) );
},
function():void {
	var s:Sprite = ExecLayout.findByName( sp_codes, "codeSBTop" );
	s.removeChild( s.getChildByName("hiliter") );
},
Anim.FADE);


// HIGHLIGHT STORE INSNS
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	var mid:Sprite = ExecLayout.findByName( sp_codes, "codeSBMiddle" );
	var h:Sprite = drawHiliter( 3 );
	h.alpha = 0.0;
	h.name = "hiliter";
	mid.addChildAt( h, 0 );
	// coords in code box space
	h.x = h.y = 0;
	g_timeline.addTween( ExecLayout.fadeToIncr(h,1.0) );
},
function():void {
	var s:Sprite = ExecLayout.findByName( sp_codes, "codeSBMiddle" );
	s.removeChild( s.getChildByName("hiliter") );
},
Anim.FADE);


// SPLIT SYNC+BURST CODE
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var top:Sprite = ExecLayout.findByName( sp_codes, "codeSBTop" );
	g_timeline.addTween( new Tween(top,"y",top.y-CODE_SPLIT_VGAP,
		Anim.Start(),Anim.StopNoIncr(Anim.SPLIT_QUANTUM),Transitions.CUBIC_IN_AND_OUT) );

	var mid:Sprite = ExecLayout.findByName( sp_codes, "codeSBMiddle" );
	g_timeline.addTween( new Tween(mid,"y",mid.y+CODE_SPLIT_VGAP,
		Anim.Start(),Anim.StopNoIncr(Anim.SPLIT_QUANTUM),Transitions.CUBIC_IN_AND_OUT) );

	var bottom:Sprite = ExecLayout.findByName( sp_codes, "codeSBBottom" );
	g_timeline.addTween( new Tween(bottom,"y",bottom.y+(3*CODE_SPLIT_VGAP),
		Anim.Start(),Anim.StopNoIncr(Anim.SPLIT_QUANTUM),Transitions.CUBIC_IN_AND_OUT) );
		
	Anim.StopIncr(Anim.SPLIT_QUANTUM);
},
function():void {},
Anim.SPLIT_QUANTUM);


			SceneManager.noMoreScenes();
		} // end init()
		
}}