/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package  {
	
	import com.boostworthy.geom.Path;
	import com.dncompute.graphics.*;
	import flash.display.*;
	import flash.events.*;
	import flash.text.*;
	import flash.ui.*;
	import com.boostworthy.animation.management.*;
	import com.boostworthy.animation.easing.*;
	import com.boostworthy.animation.sequence.*;
	import com.boostworthy.animation.sequence.tweens.*;
	import com.boostworthy.animation.rendering.*;
	import com.boostworthy.core.Global;
	
	import fl.controls.DataGrid;
	import fl.controls.dataGridClasses.*;
	
	/**
	 * Animation for DMP Sharing Table execution.
	 * @author Joe Devietti
	 */
	public class ShtabExec extends Sprite {
		
	public function ShtabExec() {
		if (stage) init();
		else addEventListener(Event.ADDED_TO_STAGE, init);
	}
	
	/** calc coords for a given row in the shtab */
	private function hiliteShtabRowCoords(row:int):Object {
		return {x:SHTAB_X,
		y:SHTAB_Y + (row * SHTAB_ROW_HEIGHT) };
	}
	/** highlight the shtab row for the specified memory location */
	private function hiliteShtabRow(location:String):void {
		// map memory locations to rows. row 0 is the header
		var map:Object = { A:1, B:2, P:3, Q:4 };
		
		g_timeline.addTween( new Tween(sp_shtab_hiliter, "y", hiliteShtabRowCoords(map[location]).y,
			Anim.Start(), Anim.StopIncr(Anim.HILITE_MOVE), Transitions.CUBIC_IN_AND_OUT) );
	}
	
	private function maskInsn(gr:Graphics,proc:int,pos:int):void {
		var o:Object = ExecLayout.hiliteInsnCoords( proc, pos );
		gr.lineStyle();
		gr.beginFill( 0x0 );
		gr.drawRect( o.x, o.y, MyShapes.INSN_WIDTH, MyShapes.INSN_HEIGHT );
		gr.endFill();
	}
	private function setupInsnHiliter(hiliter:Sprite):void {
		var mask:Sprite = new Sprite;
		maskInsn( mask.graphics, g_current_insn_hiliter_proc, g_current_insn_hiliter_pos );

		g_insn_hiliter_mask = mask;
		addChild( mask );
		
		hiliter.mask = mask;
	}
	private const TOP:uint = 0;
	private const BOTTOM:uint = 1;
	private var g_current_insn_hiliter_proc:uint = 999;
	private var g_current_insn_hiliter_pos:uint = TOP;
	private var g_insn_hiliter_mask:Sprite = null;
	
	/** Move the insn hiliter to the specified proc and top/bottom insn slot */
	private function insnHiliterTo(proc:uint, pos:uint):void {		
		// create new mask
		var mask:Sprite = new Sprite;
		
		// make current position visible
		maskInsn( mask.graphics, g_current_insn_hiliter_proc, g_current_insn_hiliter_pos );
		
		// make new position visible
		maskInsn( mask.graphics, proc, pos );
		
		// replace mask
		var old_mask:DisplayObject = sp_insn_hiliter.mask;
		sp_insn_hiliter.mask = null;
		
		// putting mask on display list makes it easier to implement insnHiliterDown()
		removeChild( g_insn_hiliter_mask );
		g_insn_hiliter_mask = mask;
		addChild( mask );

		// extend width so that p2 -> p0 transition looks like the others
		if ( g_current_insn_hiliter_proc == 2 && proc == 0 ) {
			var a:Object = ExecLayout.hiliteInsnCoords(1,0);
			sp_insn_hiliter.x = a.x;
			sp_insn_hiliter.y = a.y;
			var b:Object = ExecLayout.hiliteInsnCoords(2,1);
			sp_insn_hiliter.width = b.x + MyShapes.INSN_WIDTH - a.x;

			// reset width at end of animation
			var start:int = Anim.Start() + Anim.INSN_HILITE_MOVE - 1;
			g_timeline.addTween( new Tween(sp_insn_hiliter,"width",MyShapes.INSN_WIDTH,
				start,start+1,Transitions.LINEAR) );
		}
		
		sp_insn_hiliter.mask = mask;
		
		g_timeline.addTween( new Tween(sp_insn_hiliter,"x",ExecLayout.hiliteInsnCoords(proc,pos).x,
			Anim.Start(),Anim.StopIncr(Anim.INSN_HILITE_MOVE),Transitions.LINEAR) );
		
		g_current_insn_hiliter_proc = proc;
		g_current_insn_hiliter_pos = pos;
	}
	private function insnHiliterDown():void {
		g_timeline.addTween( new Tween( g_insn_hiliter_mask, "y", g_insn_hiliter_mask.y + MyShapes.INSN_HEIGHT, 
			Anim.Start(), Anim.StopIncr(Anim.INSN_HILITE_MOVE), Transitions.CUBIC_IN_AND_OUT) );
		g_current_insn_hiliter_pos = BOTTOM;
	}
	
	public static const SHTAB_ROW_HEIGHT:int = 30;
	private static const SHTAB_LOCATION_COL_WIDTH:int = 57;
	private static const SHTAB_OWNER_COL_WIDTH:int = 126;
	public static const SHTAB_WIDTH:int = SHTAB_LOCATION_COL_WIDTH + SHTAB_OWNER_COL_WIDTH;
	private const SHTAB_X:int = ExecLayout.COLUMN_START[0] + ExecLayout.CODE_GAP - 10;
	private const SHTAB_Y:int = 225;

	private const g_timeline:Timeline = new Timeline( RenderMethod.ENTER_FRAME );
	private var sp_code_cpus:Array = [];
	private var sp_shtab_hiliter:Sprite = null;
	private var sp_insn_hiliter:Sprite = null;
	
	private function init(e:Event = null):void {
		removeEventListener(Event.ADDED_TO_STAGE, init);
		// entry point
		
		SceneManager.setup( g_timeline, stage );
		
		// SCENE 0: draw code, cpus and token
		
		// local iteration vars
		var i:int = 0;
		var s:Sprite = null; var t:Sprite = null;
		
		var sp_token:Sprite = null;
		var sp_time_arrows:Array = [];
		
		for ( i = 0; i < ExecLayout.NUM_CPUS; i++ ) {
			// draw codebox
			s = MyShapes.CodeBox( ExecLayout.INSNS[i] );
			s.x = ExecLayout.COLUMN_START[i] + ExecLayout.CODE_GAP;
			s.y = ExecLayout.CODE_TOP;
			s.name = "codebox"+i;
			addChild( s );
			sp_code_cpus.push( s );
			
			// draw cpu
			s = MyShapes.Cpu( "p"+i );
			s.x = ExecLayout.COLUMN_START[i] + ExecLayout.SHTAB_CPU_GAP;
			s.y = ExecLayout.CPU_TOP;
			addChild( s );
			sp_code_cpus.push( s );
		}
		
		// draw token at p0
		ExecLayout.USE_SHTAB_CPU_MODE = true;
		sp_token = MyShapes.Token();
		sp_token.x = ExecLayout.tokenAt(0).x;
		sp_token.y = ExecLayout.tokenAt(0).y;
		ExecLayout.g_current_token_cpu = 0;
		addChild( sp_token );
		
		for ( i = 1; i <= 2; i++ ) {
			s = MyShapes.TimeArrow();
			s.x = ExecLayout.COLUMN_MIDPOINT_START[i] + (s.width/2);
			s.y = ExecLayout.INSN_TOP;
			addChild( s );
			sp_time_arrows.push( s );
		}


// SHOW SHARING TABLE
var sp_shtab:DataGrid = null;
var sp_shtab_label:Sprite = null;
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// show sharing table
	sp_shtab = new DataGrid();
	var location:DataGridColumn = sp_shtab.addColumn("mem");
	var owner:DataGridColumn = sp_shtab.addColumn("owner");
	
	sp_shtab.addItem({mem:"A", owner:"p0"});
	sp_shtab.addItem({mem:"B", owner:"shared"});
	sp_shtab.addItem({mem:"P", owner:"p0"});
	sp_shtab.addItem({mem:"Q", owner:"p2" });
	
	location.width = SHTAB_LOCATION_COL_WIDTH;
	owner.width = SHTAB_OWNER_COL_WIDTH;
	
	owner.cellRenderer = CellStrikethroughRenderer;
	
	sp_shtab.width = SHTAB_WIDTH;
	sp_shtab.headerHeight = SHTAB_ROW_HEIGHT;
	sp_shtab.rowHeight = SHTAB_ROW_HEIGHT;
	sp_shtab.rowCount = sp_shtab.length; // no vertical scroll bar
	sp_shtab.move( SHTAB_X, SHTAB_Y );
	sp_shtab.alpha = 0.0;
	addChild( sp_shtab );

	// style the data grid
	var cellFormat:TextFormat = new TextFormat();
	cellFormat.font = "Tw Cen MT";
	cellFormat.size = 21;
	cellFormat.bold = false;
	cellFormat.align = TextFormatAlign.CENTER;
	sp_shtab.setRendererStyle( "textFormat", cellFormat );
	sp_shtab.setRendererStyle( "textPadding", 0 );
	var headerFormat:TextFormat = new TextFormat();
	headerFormat.font = "Tw Cen MT";
	headerFormat.size = 24;
	headerFormat.bold = false;

	headerFormat.align = TextFormatAlign.CENTER; // NB: doesn't work...
	owner.headerText = "    owner"; // ...so we hack it in!
	
	sp_shtab.setStyle( "headerTextFormat", headerFormat );
	
	g_timeline.addTween( new Tween( sp_shtab, "alpha", 1.0, 
		Anim.Start(), Anim.StopIncr(Anim.FADE), Transitions.LINEAR) );
		
	// fade all and label shtab
	for each ( s in sp_code_cpus.concat(sp_token,sp_time_arrows) ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,ExecLayout.ALPHA_DIM) );
	}
	Anim.StopIncr(Anim.FADE);
	
	sp_shtab_label = MyShapes.Label( sp_shtab, "Sharing Table", MyShapes.LABEL_RIGHT );
	sp_shtab_label.x = SHTAB_X + SHTAB_WIDTH + MyShapes.LABEL_OFFSET;
	sp_shtab_label.y = SHTAB_Y;
	sp_shtab_label.alpha = 0.0;
	addChild( sp_shtab_label );
	g_timeline.addTween( ExecLayout.fadeToIncr(sp_shtab_label,1.0) );
},
function():void {
	removeChild( sp_shtab );
	sp_shtab = null;
	removeChild( sp_shtab_label );
	sp_shtab_label = null;
},
3*Anim.FADE);


// SHOW CPU STATUS INDICATORS
var sp_status:Array = [];
var sp_status_label:Sprite = null;
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	// hide shtab label and dim shtab
	g_timeline.addTween( ExecLayout.fadeToNoIncr(sp_shtab,ExecLayout.ALPHA_DIM) );
	g_timeline.addTween( ExecLayout.fadeToNoIncr(sp_shtab_label,0.0) );
	Anim.StopIncr(Anim.FADE);
	
	// add status indicators
	for ( var i:int = 0; i < ExecLayout.NUM_CPUS; i++ ) {
		var s:Sprite = MyShapes.CpuStatus( MyShapes.CPU_STATUS_PARALLEL_MODE );
		s.x = ExecLayout.COLUMN_START[i] + ExecLayout.SHTAB_CPU_GAP + MyShapes.CPU_WIDTH + 8;
		s.y = ExecLayout.CPU_TOP + ((MyShapes.CPU_WIDTH - s.height)/2.0);
		s.alpha = 0.0;
		s.name = "status"+i;
		addChild( s );
		sp_status.push( s );
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	}
	Anim.StopIncr(Anim.FADE);
	
	// add status label
	sp_status_label = MyShapes.Label( sp_status[0], "CPU Status", MyShapes.LABEL_RIGHT );
	sp_status_label.x = sp_status[0].x + sp_status[0].width + MyShapes.LABEL_OFFSET;
	sp_status_label.y = sp_status[0].y;
	sp_status_label.alpha = 0.0;
	addChild( sp_status_label );
	g_timeline.addTween( ExecLayout.fadeToIncr(sp_status_label,1.0) );
},
function():void {
	for each ( var s:Sprite in sp_status ) { removeChild(s); }
	sp_status = [];
	removeChild( sp_status_label );
	sp_status_label = null;
},
3*Anim.FADE);


// HIDE STATUS LABEL AND UNFADE ALL
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// hide shtab label and unfade everything else
	for each ( s in sp_code_cpus.concat(sp_token,sp_time_arrows,sp_shtab) ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	}
	g_timeline.addTween( ExecLayout.fadeToNoIncr(sp_status_label,0.0) );
	Anim.StopIncr(Anim.FADE);
},
function():void {},
Anim.FADE);


/*
// TEST CPU STATUS INDICATORS
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	for ( var i:int = 0; i < ExecLayout.NUM_CPUS; i++ ) {
		var anims:Array = MyShapes.updateCpuStatus( sp_status[i], MyShapes.CPU_STATUS_WAITING_FOR_TOKEN );
		for each ( var t:ITween in anims ) {
			g_timeline.addTween( t );
		}
	}
},
function():void {},
Anim.FADE*3*2);
*/

// CYCLE 1A FOR P0: highlight current insn
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var unused_hiliters:Array = [];
	
	// highlight first insns of each cpu
	for ( var i:int = 0; i < ExecLayout.NUM_CPUS; i++ ) {
		var insn_hl:Sprite = MyShapes.InsnHighlighter( (0 == i) ? 2 : 1 );
		var coords:Object = ExecLayout.hiliteInsnCoords( i, 0 );
		insn_hl.x = coords.x; insn_hl.y = coords.y;
		insn_hl.alpha = 0.0;
		insn_hl.name = "hiliter"+i;
		addChild( insn_hl );
		if ( 0 == i ) {
			sp_insn_hiliter = insn_hl;
		} else {
			unused_hiliters.push( insn_hl );
		}
		g_timeline.addTween( new Tween( insn_hl, "alpha", 1.0, 
			Anim.Start(), Anim.StopNoIncr(Anim.HILITE_FADE), Transitions.LINEAR) );
	}
	Anim.StopIncr(Anim.HILITE_FADE);
	
	// move token to top of rendering stack to avoid it getting discolored by insn hiliter
	setChildIndex( sp_token, numChildren-1 );
	
	Anim.StopIncr(15); // pause
	
	// remove hiliters for p1 and p2
	for each ( s in unused_hiliters ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(s,0.0) );
	}
	Anim.StopIncr( Anim.FADE );
	
	// setup the real insn hiliter (currently at p0)
	g_current_insn_hiliter_proc = 0;
	g_current_insn_hiliter_pos = TOP;
	setupInsnHiliter( sp_insn_hiliter );
},
function():void {
	removeChild( sp_insn_hiliter );
	sp_insn_hiliter = null;
},
Anim.HILITE_FADE + Anim.FADE + 15);


// CYCLE 1B FOR P0: check shtab
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	
	// highlight relevant row of shtab
	var s:Sprite = MyShapes.ShtabHighlighter();
	var coords:Object = hiliteShtabRowCoords( 1 );
	s.x = coords.x; s.y = coords.y;
	s.alpha = 0.0;
	addChild( s );
	sp_shtab_hiliter = s;
	g_timeline.addTween( new Tween( s, "alpha", 1.0, 
		Anim.Start(), Anim.StopIncr(Anim.HILITE_FADE), Transitions.LINEAR) );
},
function():void {
	removeChild( sp_shtab_hiliter );
	sp_shtab_hiliter = null;
},
Anim.HILITE_FADE);


// CYCLE 1B FOR P0: exec insn
var sp_insns:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// exec p0's 1st insn
	var cpu:int = 0;
	s = MyShapes.Insn( ExecLayout.INSNS[cpu][0], MyShapes.INSN_COLORS[cpu] );
	s.x = ExecLayout.COLUMN_START[cpu] + ExecLayout.INSN_GAP;
	s.y = ExecLayout.INSN_TOP;
	s.alpha = 0.0;
	s.name = "i00"
	addChild( s );
	sp_insns.push( s );
	g_timeline.addTween( new Tween( s, "alpha", 1.0, 
		Anim.Start(), Anim.StopIncr(Anim.INSN_APPEAR), Transitions.LINEAR) );
},
function():void {
	for each ( var s:Sprite in sp_insns ) { removeChild(s); }
	sp_insns = [];
},
Anim.INSN_APPEAR);


// CYCLE 1A FOR P1: show p1 being stuck
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	insnHiliterTo( 1, TOP );
	hiliteShtabRow( "B" ); // hilite shtab for p1
},
function():void {},
Anim.INSN_HILITE_MOVE+Anim.HILITE_MOVE);


// CYCLE 1B FOR P1: p1 is done with parallel mode
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var anims:Array = MyShapes.updateCpuStatus( sp_status[1], MyShapes.CPU_STATUS_PARALLEL_MODE_FINISHED );
	for each ( var t:ITween in anims ) {
		g_timeline.addTween( t );
	}
},
function():void {},
2*Anim.CPU_STATUS_FADE);


// CYCLE 1A FOR P2: check sharing table
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	insnHiliterTo( 2, TOP );
	hiliteShtabRow( "Q" ); // hilite shtab for p2
},
function():void {},
Anim.INSN_HILITE_MOVE+Anim.HILITE_MOVE);


// CYCLE 1B FOR P2: exec insn
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	// exec p2's 1st insn
	var cpu:int = 2;
	var s:Sprite = MyShapes.Insn( ExecLayout.INSNS[cpu][0], MyShapes.INSN_COLORS[cpu] );
	s.x = ExecLayout.COLUMN_START[cpu] + ExecLayout.INSN_GAP;
	s.y = ExecLayout.INSN_TOP;
	s.alpha = 0.0;
	s.name = "i20";
	addChild( s );
	sp_insns.push( s );
	g_timeline.addTween( new Tween( s, "alpha", 1.0, Anim.Start(), Anim.StopIncr(Anim.INSN_APPEAR), Transitions.LINEAR) );
},
function():void {
	sp_insns = ExecLayout.removeByName( ["i20"], sp_insns, removeChild );
},
Anim.INSN_APPEAR);


// CYCLE 2A FOR P0: check shtab
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	insnHiliterTo( 0, TOP );

	// move hiliter to p0 insn 1
	insnHiliterDown();
	
	// hilite shtab for p0
	hiliteShtabRow( "P" );
},
function():void {},
(2*Anim.INSN_HILITE_MOVE) + (Anim.HILITE_MOVE));


// CYCLE 2B FOR P0: exec insn
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	// exec p0's 2nd insn
	var cpu:int = 0;
	var s:Sprite = MyShapes.Insn( ExecLayout.INSNS[cpu][1], MyShapes.INSN_COLORS[cpu] );
	s.x = ExecLayout.COLUMN_START[cpu] + ExecLayout.INSN_GAP;
	s.y = ExecLayout.INSN_TOP + MyShapes.INSN_YOFFSET;
	s.alpha = 0.0;
	s.name = "i01";
	addChild( s );
	sp_insns.push( s );
	g_timeline.addTween( new Tween( s, "alpha", 1.0, 
		Anim.Start(), Anim.StopIncr(Anim.INSN_APPEAR), Transitions.LINEAR) );
},
function():void {
	sp_insns = ExecLayout.removeByName( ["i01"], sp_insns, removeChild );
},
Anim.INSN_APPEAR);


// CYCLE 2C FOR P0: p0 is done with parallel mode
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var anims:Array = MyShapes.updateCpuStatus( sp_status[0], MyShapes.CPU_STATUS_PARALLEL_MODE_FINISHED );
	for each ( var t:ITween in anims ) {
		g_timeline.addTween( t );
	}
},
function():void {},
2*Anim.CPU_STATUS_FADE);


// CYCLE 2 FOR P1
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// move insn hiliter
	insnHiliterTo( 1, TOP );
	
	// hilite shtab for p1
	hiliteShtabRow( "B" );
},
function():void {},
(Anim.INSN_HILITE_MOVE) + (Anim.HILITE_MOVE));


// CYCLE 2B FOR P2: p2 is done with parallel mode
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	insnHiliterTo( 2, TOP );
	insnHiliterDown();
	
	var anims:Array = MyShapes.updateCpuStatus( sp_status[2], MyShapes.CPU_STATUS_PARALLEL_MODE_FINISHED );
	for each ( var t:ITween in anims ) {
		g_timeline.addTween( t );
	}
},
function():void {},
(2*Anim.INSN_HILITE_MOVE) + (2*Anim.CPU_STATUS_FADE));


// TRANSITION P0 TO SERIAL MODE
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	insnHiliterTo( 0, BOTTOM );
	var anims:Array = MyShapes.updateCpuStatus( sp_status[0], MyShapes.CPU_STATUS_SERIAL_MODE );
	for each ( var t:ITween in anims ) {
		g_timeline.addTween( t );
	}
},
function():void {},
Anim.INSN_HILITE_MOVE + (2*Anim.CPU_STATUS_FADE));


// PASS TOKEN TO P1
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	// pass token to p1
	g_timeline.addTween( ExecLayout.moveTokenTo(sp_token,1) );
	
	// move insn hiliter
	insnHiliterTo( 1, TOP );
},
function():void {},
(Anim.INSN_HILITE_MOVE) + Anim.MOVE_TOKEN);


// TRANSITION P1 TO SERIAL MODE
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	var anims:Array = MyShapes.updateCpuStatus( sp_status[1], MyShapes.CPU_STATUS_SERIAL_MODE );
	for each ( var t:ITween in anims ) {
		g_timeline.addTween( t );
	}
},
function():void {},
(2*Anim.CPU_STATUS_FADE));


// EXEC P1'S FIRST INSN
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// update shtab for p1
	hiliteShtabRow( "B" );
		
	g_timeline.addTween( new Action(sp_shtab.editField, [1,"owner","shared p1"], 
		sp_shtab.editField, [1,"owner","shared"], Anim.Start() ) );
	
	// exec p1's 1st insn
	var cpu:int = 1;
	var s:Sprite = MyShapes.Insn( ExecLayout.INSNS[cpu][0], MyShapes.INSN_COLORS[cpu] );
	s.x = ExecLayout.COLUMN_START[cpu] + ExecLayout.INSN_GAP;
	s.y = ExecLayout.INSN_TOP + (2*MyShapes.INSN_YOFFSET);
	s.alpha = 0.0;
	s.name = "i10";
	addChild( s );
	sp_insns.push( s );
	g_timeline.addTween( new Tween( s, "alpha", 1.0, Anim.Start(), Anim.StopIncr(Anim.INSN_APPEAR), Transitions.LINEAR) );
	
},
function():void {
	sp_insns = ExecLayout.removeByName( ["i10"], sp_insns, removeChild );
},
Anim.HILITE_MOVE+Anim.INSN_APPEAR);


// P1'S SECOND INSN
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	
	// move hiliter to p1 insn 1
	insnHiliterDown();
	
	// change shtab for p1
	hiliteShtabRow( "A" );
	
	g_timeline.addTween( new Action(sp_shtab.editField, [0,"owner","p0 p1"], 
		sp_shtab.editField, [0,"owner","p0"], Anim.Start() ) );
	
	// exec p1's 2nd insn
	var cpu:int = 1;
	var s:Sprite = MyShapes.Insn( ExecLayout.INSNS[cpu][1], MyShapes.INSN_COLORS[cpu] );
	s.x = ExecLayout.COLUMN_START[cpu] + ExecLayout.INSN_GAP;
	s.y = ExecLayout.INSN_TOP + (3*MyShapes.INSN_YOFFSET);
	s.alpha = 0.0;
	s.name = "i11";
	addChild( s );
	sp_insns.push( s );
	g_timeline.addTween( new Tween( s, "alpha", 1.0, Anim.Start(), Anim.StopIncr(Anim.INSN_APPEAR), Transitions.LINEAR) );
},
function():void {
	sp_insns = ExecLayout.removeByName( ["i11"], sp_insns, removeChild );
},
Anim.INSN_HILITE_MOVE + Anim.HILITE_MOVE + Anim.INSN_APPEAR);


// PASS TOKEN TO P2
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// pass token to p2
	g_timeline.addTween( ExecLayout.moveTokenTo(sp_token,2) );
	// move insn hiliter
	insnHiliterTo( 2, BOTTOM );

// TRANSITION P2 TO SERIAL MODE
	var anims:Array = MyShapes.updateCpuStatus( sp_status[2], MyShapes.CPU_STATUS_SERIAL_MODE );
	for each ( var t:ITween in anims ) {
		g_timeline.addTween( t );
	}

// EXEC P2'S SECOND INSN
	// change shtab for p2
	hiliteShtabRow( "B" );
		
	g_timeline.addTween( new Action(sp_shtab.editField, [1,"owner","shared p1 p2"], 
		sp_shtab.editField, [1,"owner","shared p1"], Anim.Start() ) );
	
	// exec p2's 2nd insn
	var cpu:int = 2;
	var s:Sprite = MyShapes.Insn( ExecLayout.INSNS[cpu][1], MyShapes.INSN_COLORS[cpu] );
	s.x = ExecLayout.COLUMN_START[cpu] + ExecLayout.INSN_GAP;
	s.y = ExecLayout.INSN_TOP + (4*MyShapes.INSN_YOFFSET);
	s.alpha = 0.0;
	s.name = "i21";
	addChild( s );
	sp_insns.push( s );
	g_timeline.addTween( new Tween( s, "alpha", 1.0, 
		Anim.Start(), Anim.StopIncr(Anim.INSN_APPEAR), Transitions.LINEAR) );
},
function():void {
	sp_insns = ExecLayout.removeByName( ["i21"], sp_insns, removeChild );
},
Anim.INSN_HILITE_MOVE+Anim.MOVE_TOKEN + (2*Anim.CPU_STATUS_FADE) + Anim.HILITE_MOVE+Anim.INSN_APPEAR );


// SHOW PARALLEL PREFIX
var sp_parprefix_hiliter:Sprite = null;
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// fade all visible stuff
	for each ( var d:DisplayObject in sp_time_arrows.concat(sp_code_cpus,sp_status,sp_token,sp_insns) ) {
		// filter out parallel mode insns
		if ( d.name!="i00" && d.name!="i01" && d.name!="i20" ) {
			g_timeline.addTween( ExecLayout.fadeToNoIncr(d,ExecLayout.ALPHA_DIM) );
		}
	}
	// make shtab and hiliters go away completely
	for each ( d in [sp_shtab,sp_shtab_hiliter,sp_insn_hiliter] ) {
		g_timeline.addTween( ExecLayout.fadeToNoIncr(d,0.0) );
	}
	Anim.StopIncr(Anim.FADE);
	
	// draw box around parallel insns
	sp_parprefix_hiliter = new Sprite();
	sp_parprefix_hiliter.graphics.lineStyle( 2, 0x0 );
	var left:Sprite = ExecLayout.findByName( sp_insns, "i00" );
	var right:Sprite = ExecLayout.findByName( sp_insns, "i20" );
	sp_parprefix_hiliter.graphics.drawRect( 0, 0, 
		(right.x+right.width) - left.x + (2*MyShapes.XACT_XOFFSET), 
		(2*MyShapes.INSN_YOFFSET) );
	sp_parprefix_hiliter.alpha = 0.0;
	
	// create text
	var tf:TextField = new UnselectableTextField();
	tf.text = "Parallel Prefix";
	tf.autoSize = TextFieldAutoSize.CENTER;
	
	// format text
	var format:TextFormat = new TextFormat();
	format.font = "Tw Cen MT";
	format.color = 0x0;
	format.size = 40;
	format.bold = false;
	tf.setTextFormat( format );
	tf.x = 0;
	tf.y = 0;
	sp_parprefix_hiliter.addChild( tf );
	tf.x = ((sp_parprefix_hiliter.width - tf.width) / 2.0);
	tf.y = (sp_parprefix_hiliter.height - tf.height) / 2.0;
	
	addChild( sp_parprefix_hiliter );
	sp_parprefix_hiliter.x = ExecLayout.COLUMN_START[0] + ExecLayout.INSN_GAP - MyShapes.XACT_XOFFSET;
	sp_parprefix_hiliter.y = ExecLayout.INSN_TOP - (MyShapes.XACT_YOFFSET/2);
	g_timeline.addTween( ExecLayout.fadeToIncr(sp_parprefix_hiliter,1.0) );
},
function():void {
	removeChild( sp_parprefix_hiliter );
	sp_parprefix_hiliter = null;
},
Anim.FADE*2);
	

// SHOW SERIAL SUFFIX
var sp_sersuffix_hiliter:Sprite = null;
SceneManager.addScene(
function(start_frame:uint, end_frame:uint):void {
	// fade parallel box
	//g_timeline.addTween( ExecLayout.fadeToIncr(sp_parprefix_hiliter,0.0) );
	
	// highlight only serial suffix insns
	for each ( var d:DisplayObject in sp_insns ) {
		if ( d.name=="i10" || d.name=="i11" || d.name=="i21" ) { // serial mode insns
			g_timeline.addTween( ExecLayout.fadeToNoIncr(d,1.0) );
		} else { // parallel mode insns
			//g_timeline.addTween( ExecLayout.fadeToNoIncr(d,ExecLayout.ALPHA_DIM) );
		}
	}
	Anim.StopIncr(Anim.FADE);
	
	// draw box around serial insns
	sp_sersuffix_hiliter = new Sprite();
	sp_sersuffix_hiliter.graphics.lineStyle( 2, 0x0 );
	var left:Sprite = ExecLayout.findByName( sp_insns, "i00" );
	var right:Sprite = ExecLayout.findByName( sp_insns, "i20" );
	sp_sersuffix_hiliter.graphics.drawRect( 0, 0, 
		(right.x + right.width) - left.x + (2*MyShapes.XACT_XOFFSET), 
		(3*MyShapes.INSN_YOFFSET) );
	sp_sersuffix_hiliter.alpha = 0.0;
	
	// create text
	var tf:TextField = new UnselectableTextField();
	tf.text = "Serial Suffix";
	tf.autoSize = TextFieldAutoSize.CENTER;
	
	// format text
	var format:TextFormat = new TextFormat();
	format.font = "Tw Cen MT";
	format.color = 0x0;
	format.size = 40;
	format.bold = false;
	tf.setTextFormat( format );
	tf.x = 0;
	tf.y = 0;
	sp_sersuffix_hiliter.addChild( tf );
	// don't overlap p1's insns
	tf.x = 7;
	tf.y = (sp_sersuffix_hiliter.height - tf.height) / 2.0;
	
	addChild( sp_sersuffix_hiliter );
	sp_sersuffix_hiliter.x = ExecLayout.COLUMN_START[0] + ExecLayout.INSN_GAP - MyShapes.XACT_XOFFSET;
	sp_sersuffix_hiliter.y = ExecLayout.findByName(sp_insns,"i10").y - (MyShapes.XACT_YOFFSET/2);
	g_timeline.addTween( ExecLayout.fadeToIncr(sp_sersuffix_hiliter,1.0) );
},
function():void {
	removeChild( sp_sersuffix_hiliter );
	sp_sersuffix_hiliter = null;
},
Anim.FADE*2);


// SHOW COMMUNICATION ARROWS
var sp_comm_arrows:Array = [];
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {
	// fade serial box
	//g_timeline.addTween( ExecLayout.fadeToIncr(sp_sersuffix_hiliter,0.0) );
	
	// undim all insns
	//for each ( var s:Sprite in sp_insns ) {
	//	g_timeline.addTween( ExecLayout.fadeToNoIncr(s,1.0) );
	//}
	//Anim.StopIncr(Anim.FADE);
	
	// wipe in arrows
	var from:Sprite = ExecLayout.findByName( sp_insns, "i00" );
	var to:Sprite = ExecLayout.findByName( sp_insns, "i11" );
	var arrow:Sprite = new TextLineArrow( from.x+from.width - ExecLayout.COMM_ARROW_XOFFSET, from.y+(from.height/2), 
		to.x + ExecLayout.COMM_ARROW_XOFFSET, to.y+(to.height/2), "RAW", ExecLayout.COMM_ARROW_COLOR );
	addChild( arrow );
	sp_comm_arrows.push( arrow );
	g_timeline.addTween( ExecLayout.pathWipeTLtoBR(arrow, Anim.COMM_ARROW_WIPE) );
		
	from = ExecLayout.findByName( sp_insns, "i10" );
	to = ExecLayout.findByName( sp_insns, "i21" );
	arrow = new TextLineArrow( from.x+from.width - ExecLayout.COMM_ARROW_XOFFSET, from.y+(from.height/2), 
		to.x + ExecLayout.COMM_ARROW_XOFFSET, to.y+(to.height/2), "WAW", ExecLayout.COMM_ARROW_COLOR );
	addChild( arrow );
	sp_comm_arrows.push( arrow );
	g_timeline.addTween( ExecLayout.pathWipeTLtoBR(arrow, Anim.COMM_ARROW_WIPE) );		
},
function():void {
	for each ( var s:Sprite in sp_comm_arrows ) { removeChild(s); }
	sp_comm_arrows = [];
},
(2*Anim.COMM_ARROW_WIPE));


/*
// DESCRIPTION
SceneManager.addScene( 
function(start_frame:uint, end_frame:uint):void {

},
function():void {

},
XXX);
*/

	SceneManager.noMoreScenes();

	} // end init()
	
}}